/*

  sshmp-powm.c

  Author: Mika Kojo <mkojo@ssh.fi>

  Copyright (c) 2000 SSH Communications Security Corp
  All rights reserved.

  Created Sun Feb 18 22:54:16 2001.

  */

#include "sshincludes.h"
#include "sshmp.h"

void ssh_mprz_powm(SshMPInteger ret, SshMPIntegerConst g, SshMPIntegerConst e,
                   SshMPIntegerConst p)
{
  SshMPIntIdealStruct m;
  SshMPIntModStruct gm;

  ssh_mprzm_init_ideal(&m, p);
  ssh_mprzm_init(&gm, &m);

  ssh_mprzm_set_mprz(&gm, g);
  ssh_mprzm_pow(&gm, &gm, e);
  ssh_mprz_set_mprzm(ret, &gm);
  
  ssh_mprzm_clear(&gm);
  ssh_mprzm_clear_ideal(&m);
}

void ssh_mprz_powm_gg(SshMPInteger ret,
                      SshMPIntegerConst g1, SshMPIntegerConst e1, 
                      SshMPIntegerConst g2, SshMPIntegerConst e2,
                      SshMPIntegerConst p)
{
  SshMPIntIdealStruct m;
  SshMPIntModStruct g1m, g2m;

  ssh_mprzm_init_ideal(&m, p);
  ssh_mprzm_init(&g1m, &m);
  ssh_mprzm_init(&g2m, &m);

  ssh_mprzm_set_mprz(&g1m, g1);
  ssh_mprzm_set_mprz(&g2m, g2);
  ssh_mprzm_pow_gg(&g1m, &g1m, e1, &g2m, e2);
  ssh_mprz_set_mprzm(ret, &g1m);
  
  ssh_mprzm_clear(&g1m);
  ssh_mprzm_clear(&g2m);
  ssh_mprzm_clear_ideal(&m);
}

void ssh_mprz_powm_ui_g(SshMPInteger ret, SshWord g, SshMPIntegerConst e,
                        SshMPIntegerConst p)
{
  SshMPIntIdealStruct m;
  SshMPIntModStruct gm;

  ssh_mprzm_init_ideal(&m, p);
  ssh_mprzm_init(&gm, &m);

  ssh_mprzm_pow_ui_g(&gm, g, e);
  ssh_mprz_set_mprzm(ret, &gm);
  
  ssh_mprzm_clear(&gm);
  ssh_mprzm_clear_ideal(&m);
}

void ssh_mprz_powm_ui_exp(SshMPInteger ret, SshMPIntegerConst g,
                         SshWord e, SshMPIntegerConst p)
{
  SshMPIntIdealStruct m;
  SshMPIntModStruct gm;

  ssh_mprzm_init_ideal(&m, p);
  ssh_mprzm_init(&gm, &m);
  ssh_mprzm_set_mprz(&gm, g);

  ssh_mprzm_pow_ui_exp(&gm, &gm, e);
  ssh_mprz_set_mprzm(ret, &gm);
  
  ssh_mprzm_clear(&gm);
  ssh_mprzm_clear_ideal(&m);
}

void ssh_mprz_powm_precomp_init(SshMPIntModPowPrecomp precomp,
                                SshMPIntegerConst g, SshMPIntegerConst p,
                                SshMPIntegerConst order)
{
  SshMPIntIdeal m = ssh_xcalloc(1, sizeof(*m));
  SshMPIntModStruct gm;

  ssh_mprzm_init_ideal(m, p);
  ssh_mprzm_init(&gm, m);
  ssh_mprzm_set_mprz(&gm, g);

  ssh_mprzm_pow_precomp_init(precomp, &gm, order);
  
  ssh_mprzm_clear(&gm);
}

void ssh_mprz_powm_precomp_clear(SshMPIntModPowPrecomp precomp)
{
  SshMPIntIdeal m = (SshMPIntIdeal)ssh_mprzm_pow_precomp_get_ideal(precomp);
  ssh_mprzm_pow_precomp_clear(precomp);
  ssh_mprzm_clear_ideal(m);
  ssh_xfree(m);
}

void ssh_mprz_powm_with_precomp(SshMPInteger ret, SshMPIntegerConst e,
                                SshMPIntModPowPrecompConst precomp)
{
  SshMPIntModStruct gm;

  ssh_mprzm_init(&gm, ssh_mprzm_pow_precomp_get_ideal(precomp));

  ssh_mprzm_pow_precomp(&gm, e, precomp);
  ssh_mprz_set_mprzm(ret, &gm);
  
  ssh_mprzm_clear(&gm);
}

/* sshmp-powm.c */
